/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.tomcat.jakartaee;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Used to represent a specification profile.
 */
public interface EESpecProfile {
    /**
     * Convert the specified name to the target namespace.
     * @param name the name to convert
     * @return the converted name
     */
    default String convert(String name) {
        Matcher m = getPattern().matcher(name);
        return m.replaceAll(getTarget() + "$1");
    }

    /**
     * The source namespace.
     * @return the source namespace
     */
    String getSource();

    /**
     * The target namespace.
     * @return the target namespace
     */
    String getTarget();

    /**
     * The pattern used for conversion.
     * @return the pattern
     */
    Pattern getPattern();
}

